package benefactoraddresshandler

import (
	"net/http"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/address"
	"git.gocasts.ir/ebhomengo/niki/pkg/claim"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// UpdateAddress godoc
// @Summary      Update benefactor address
// @Tags         Benefactors Addresses
// @Accept       json
// @Produce      json
// @Param        id path int true "Address ID"
// @Param        Request body   param.UpdateAddressRequest true "Edit address details"
// @Success      204
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerBenefactor
// @Router       /benefactors/addresses/{id} [put].
func (h Handler) UpdateAddress(c echo.Context) error {
	var req param.UpdateAddressRequest
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	if bErr := echo.PathParamsBinder(c).Uint("id", &req.ID).BindError(); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	claims := claim.GetClaimsFromEchoContext(c)
	req.BenefactorID = claims.UserID

	resp, sErr := h.addressSvc.Update(c.Request().Context(), req)
	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusNoContent, nil)
}
